from django import template
from django.conf import settings
from django.contrib.auth.models import User
from django.contrib.sites.models import Site
from django.core.mail import send_mail
from django.core.urlresolvers import reverse
from django.utils.http import int_to_base36
from inviter.models import OptOut
from inviter.views import import_attribute, TOKEN_GENERATOR
import shortuuid


FROM_EMAIL = getattr(settings, 'INVITER_FROM_EMAIL', settings.DEFAULT_FROM_EMAIL)

token_generator = import_attribute(TOKEN_GENERATOR)

def send_invite(invitee, inviter, url=None, opt_out_url=None, **kwargs):
    """
    Send the default invitation email assembled from 
    ``inviter/email/subject.txt`` and ``inviter/email/body.txt``
    
    Both templates will receive all the ``kwargs``.
    
    :param invitee: The invited user
    :param inviter: The inviting user
    :param url: The invite URL
    :param subject_template: The template to render for the subject
    :param body_template: The template to render for the body
    :param opt_out_url: A URL where users can permanently opt out of invitations
    """
    ctx = {'invitee': invitee, 'inviter': inviter}
    ctx.update(kwargs)
    ctx.update(site=Site.objects.get_current(), url=url)
    ctx = template.Context(ctx)
    
    subject_template = kwargs.pop('subject_template', 'inviter/email/subject.txt')
    body_template = kwargs.pop('body_template', 'inviter/email/body.txt')
    
    subject = template.loader.get_template(subject_template)
    body = template.loader.get_template(body_template)
    
    subject = subject.render(ctx)
    body = body.render(ctx)
    
    subject = ' '.join(subject.split('\n')) # No newlines in subject lines allowed
    
    send_mail(subject, body, FROM_EMAIL, [invitee.email])

def invite(email, inviter, sendfn=send_invite, **kwargs):
    """
    Invite a given email address and return a ``(User, sent)`` tuple similar
    to the Django :meth:`django.db.models.Manager.get_or_create` method.
    
    If a user with ``email`` address does not exist:
    
    * Creates a :class:`django.contrib.auth.models.User` object 
    * Set ``user.email = email``
    * Set ``user.is_active = False``
    * Set a random password
    * Send the invitation email
    * Return ``(user, True)``
    
    If a user with ``email`` address exists and ``user.is_active == False``:
    
    * Re-send the invitation 
    * Return ``(user, True)``
    
    If a user with ``email`` address exists:
    
    * Don't send  the invitation
    * Return ``(user, False)``
    
    If the email address is blocked:
    
    * Don't send the invitation
    * Return ``(None, False)``
     
    To customize sending, pass in a new ``sendfn`` function as documented by
    :attr:`inviter.utils.send_invite`:
    
    ::
    
        sendfn = lambda invitee, inviter, **kwargs: 1
        invite("foo@bar.com", request.user, sendfn = sendfn)         

    
    :param email: The email address
    :param inviter: The user inviting the email address
    :param sendfn: An email sending function. Defaults to :attr:`inviter.utils.send_invite`
    """
    
    if OptOut.objects.is_blocked(email): 
        return None, False
    try:
        user = User.objects.get(email=email)
        if user.is_active:
            return user, False
    except User.DoesNotExist:
        user = User.objects.create(
            username=shortuuid.uuid(),
            email=email,
            is_active=False
        )
        user.set_password(User.objects.make_random_password())
        user.save()
    
    url_parts = int_to_base36(user.id), token_generator.make_token(user)
    url = reverse('inviter:register', args=url_parts)
    
    opt_out_url = reverse('inviter:opt-out', args=url_parts)
    kwargs.update(opt_out_url=opt_out_url)
    
    sendfn(user, inviter, url=url, **kwargs)
    return user, True
    



from django.db import models
import hashlib

class OptOutManager(models.Manager):
    def is_blocked(self, email=None):
        """ Check if a given email address is on the block list. """
        return self.filter(hash=hashlib.sha1(email).hexdigest()).count() > 0
    
    def create(self, email=None):
        """ Create an opt out. """
        return super(OptOutManager, self).create(hash=hashlib.sha1(email).hexdigest())
    
class OptOut(models.Model):
    """ Opt-out email addresses are stored as SHA1 hashes to make sure we don't 
    accidentally collect any more data once a person signalled they're not 
    interested in receiving any more invitation emails from us. """
    hash = models.CharField(max_length=255)
    
    objects = OptOutManager()
    
    def __unicode__(self):
        return self.hash






from django import forms
from django.contrib.auth.models import User
from django.utils.translation import ugettext as _
from inviter.models import OptOut


class RegistrationForm(forms.ModelForm):
    """ The standard form that is displayed to users when registering. It gives
    a user the option to change their email address. """
    email = forms.EmailField(label=_("Email address"))
    new_password1 = forms.CharField(label=_("New password"), widget=forms.PasswordInput)
    new_password2 = forms.CharField(label=_("New password confirmation"), widget=forms.PasswordInput)

    class Meta:
        model = User
        fields = ('username', 'email')
        
    def __init__(self, *args, **kwargs):
        super(RegistrationForm, self).__init__(*args, **kwargs)
        # Hide the UUID string
        self.initial['username'] = ''

    def clean_new_password2(self):
        password1 = self.cleaned_data.get('new_password1')
        password2 = self.cleaned_data.get('new_password2')
        if password1 and password2:
            if password1 != password2:
                raise forms.ValidationError(_("The two password fields didn't match."))
        return password2

    def save(self, *args, **kwargs):
        user = super(RegistrationForm, self).save(*args, **kwargs)
        user.set_password(self.cleaned_data.get('new_password1'))
        user.is_active = True
        if kwargs.get('commit', True):
            user.save()
        return user
    

class OptOutForm(forms.ModelForm):
    """ Dummy form for opting out. """
    class Meta:
        model = User
        fields = ()
    
    def save(self):
        """ Delete the user object from the database and store the SHA1 hashed
        email address in the database to make sure this person does not receive
        any further invitation emails. """
        
        email = self.instance.email
        self.instance.delete()
        return OptOut.objects.create(email=email)



# Create your views here.
from django.conf import settings
from django.contrib.auth.models import User
from django.core.urlresolvers import reverse
from django.http import Http404, HttpResponseRedirect, HttpResponseForbidden
from django.utils import importlib
from django.utils.http import base36_to_int
from django.views.generic.base import TemplateView
from inviter.forms import OptOutForm


FORM = getattr(settings, 'INVITER_FORM', 'inviter.forms.RegistrationForm')
REDIRECT = getattr(settings, 'INVITER_REDIRECT', 'inviter:done')
TOKEN_GENERATOR = getattr(settings, 'INVITER_TOKEN_GENERATOR', 'inviter.tokens.generator')

def import_attribute(path):
    """
    Import an attribute from a module.
    """
    module = '.'.join(path.split('.')[:-1])
    function = path.split('.')[-1]

    module = importlib.import_module(module)
    return getattr(module, function)

class UserMixin(object):
    token_generator = import_attribute(TOKEN_GENERATOR)
    def get_user(self, uidb36):
        try:
            uid_int = base36_to_int(uidb36) 
            user = User.objects.get(id=uid_int)
        except (ValueError, User.DoesNotExist):
            raise Http404("No such invited user.")
        return user

class Register(UserMixin, TemplateView):
    """
    A registration view for invited users. The user model already exists - this
    view just takes care of setting a password and username, and maybe update
    the email address. Anywho - one can customize the form that is used.
    
    """
    template_name = 'inviter/register.html'
    form = import_attribute(FORM)
    redirect_url = REDIRECT

    def get(self, request, uidb36, token):
        """
        Unfortunately just a copy of 
        :attr:`django.contrib.auth.views.password_reset_confirm`
        """
        assert uidb36 is not None and token is not None
        user = self.get_user(uidb36)

        if not self.token_generator.check_token(user, token):
            return HttpResponseForbidden()
        
        return self.render_to_response({'invitee': user, 'form': self.form(instance=user)})
        
    def post(self, request, uidb36, token):
        """
        Unfortunately just a copy of 
        :attr:`django.contrib.auth.views.password_reset_confirm`
        """
        assert uidb36 is not None and token is not None
        user = self.get_user(uidb36)
        
        if not self.token_generator.check_token(user, token):
            return HttpResponseForbidden()
        
        form = self.form(request.POST, instance=user)
        
        if form.is_valid():
            form.save()
            try:
                return HttpResponseRedirect(reverse(self.redirect_url))
            except:
                return HttpResponseRedirect(self.redirect_url)
        return self.render_to_response({'invitee': user, 'form': form})

class Done(TemplateView):
    template_name = 'inviter/done.html'
    
    def get(self, request):
        return self.render_to_response({})
    
class OptOut(UserMixin, TemplateView):
    """ We want to give the user also the option to *not* receive any
    invitations anymore, which is happening in this view and 
    :class:`inviter.forms.OptOutForm`. """
    template_name = 'inviter/opt-out.html'
    
    def get(self, request, uidb36, token):
        assert uidb36 is not None and token is not None
        user = self.get_user(uidb36)

        if not self.token_generator.check_token(user, token):
            return HttpResponseForbidden()
        
        form = OptOutForm(instance=user)
        
        return self.render_to_response({'form': form})
    
    def post(self, request, uidb36, token):
        assert uidb36 is not None and token is not None
        user = self.get_user(uidb36)

        if not self.token_generator.check_token(user, token):
            return HttpResponseForbidden()
        
        form = OptOutForm(request.POST, instance=user)
        
        if form.is_valid():
            form.save()
            return HttpResponseRedirect(reverse('inviter:opt-out-done'))
        return self.render_to_response({'form': form})

class OptOutDone(TemplateView):
    template_name = 'inviter/opt-out-done.html'
    
    def get(self, request):
        return self.render_to_response({})
            
            
